package de.hhu.ba.yoshikoWrapper.core;


import java.io.IOException;

import org.cytoscape.model.CyColumn;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.work.Tunable;
import org.cytoscape.work.TunableValidator;

public class ParameterSet implements TunableValidator
{

	@Tunable(description="Network to analyze for clusters", context="nogui")
	public CyNetwork net = CyCore.cy.getCurrentNetwork();

	@Tunable(description="Time Limit for the ILP mode", context="nogui")
	public int timeLimit = -1;

	@Tunable(description="A column in the edge table containing weights", context="nogui")
	public String weightColumnName;
	@Tunable(description="A column containing boolean entries for edges that are to be treated as permanent",context="nogui")
	public String permanentColumnName;
	@Tunable(description="A column containing boolean entries for edges that are to be treated as forbidden",context="nogui")
	public String forbiddenColumnName;

	@Tunable(description="The default insertion cost that is to be used for non-existing edges",context="nogui")
	public double defaultInsertionCost = -1;
	@Tunable(description="The default deletion cost that is to be used for edges without an associated weight",context="nogui")
	public double defaultDeletionCost = 1;

	@Tunable(description="A bitmask representing which reduction rules should be used",context="nogui")
	public String reductionRulesBitMask = "000000";
	@Tunable(description="A value controlling the resolution of the SNR reduction rule. Higher values mean a longer running time but possibly better reduction.",context="nogui")
	public double snrMultFactor = 1.0;

	@Tunable(description="Alternative Callback for CPLEX, might be faster on certain instances",context="nogui")
	public boolean useTriangleCuts = false;
	@Tunable(description="Alternative Callback for CPLEX, might be faster on large instances",context="nogui")
	public boolean usePartitionCuts = false;

	@Tunable(description="Uses a heuristic instead of ILP to solve WCE, significantly faster",context="nogui")
	public boolean useHeuristic = true;

	@Tunable(description="The maximum number of (optimal) solutions that is to be calculated",context="nogui")
	public int solCount = 1;

	@Tunable(description="Disable multithreading to keep the system responsive",context="nogui")
	public boolean disableMultiThreading;

	@Tunable(description="Automatically choose an appopriate set of reduction rules (overrides a given bitmask)",context="nogui")
	/**Describes whether auto configuration of the reduction rules is to be used. Overrides the bit mask.**/
	public boolean suggestReduction = true;

	@Override
	public ValidationState getValidationState(Appendable errMsg) {
		System.out.println("DEBUG: Running VALIDATION of tunables"); //TODO: Move to logger (if it would work)
		try {
			if (net!= null) {
				//Verify column validity
				CyColumn weightColumn = net.getDefaultEdgeTable().getColumn(weightColumnName);
				if (weightColumn == null) {
					errMsg.append("Could not find a column named: "+weightColumnName+"\n");
					return ValidationState.INVALID;
				}
			}
			if (!checkBitmask(reductionRulesBitMask)) {
				errMsg.append("The Bitmask provided is invalid! Needs to be six bit binary (example: 011001)\n");
				return ValidationState.INVALID;
			}
			return ValidationState.OK;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ValidationState.INVALID;
	}

	/**
	 * Helper function that verifies a String and checks if it represents a 6-bit bitmask
	 * @param mask
	 * @return
	 */
	private boolean checkBitmask(String mask) {
		if (mask.length() != 6) {
			return false;
		}
		for (byte c : mask.getBytes()) {
			if (((char)c)!='0'&&((char)c)!='0') {
				return false;
			}
		}
		return true;
	}


}
